@@ -1,6 +1,4 @@
 /*
- * $Id$
- *
  * Copyright © 2004 Keith Packard
  *
  * Permission to use, copy, modify, distribute, and sell this software and its
@@ -21,10 +19,41 @@
  * TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
  * PERFORMANCE OF THIS SOFTWARE.
  */
+
+#ifdef HAVE_CONFIG_H
 #include <config.h>
+#endif
+
 #include <string.h>
-#include "pixman.h"
+
 #include "pixman-private.h"
+#include "pixman-accessor.h"
+
+/*
+ * Step across a small sample grid gap
+ */
+#define RenderEdgeStepSmall(edge) { \
+    edge->x += edge->stepx_small;   \
+    edge->e += edge->dx_small;	    \
+    if (edge->e > 0)		    \
+    {				    \
+	edge->e -= edge->dy;	    \
+	edge->x += edge->signdx;    \
+    }				    \
+}
+
+/*
+ * Step across a large sample grid gap
+ */
+#define RenderEdgeStepBig(edge) {   \
+    edge->x += edge->stepx_big;	    \
+    edge->e += edge->dx_big;	    \
+    if (edge->e > 0)		    \
+    {				    \
+	edge->e -= edge->dy;	    \
+	edge->x += edge->signdx;    \
+    }				    \
+}
 
 #ifdef PIXMAN_FB_ACCESSORS
 #define PIXMAN_RASTERIZE_EDGES pixman_rasterize_edges_accessors
@@ -54,10 +83,10 @@
 
 #define StepAlpha	((__ap += __ao), (__ao ^= 1))
 
-#define AddAlpha(a) {						\
-	uint8_t   __o = READ(__ap);				\
-	uint8_t   __a = (a) + Get4(__o, __ao);			\
-	WRITE(__ap, Put4 (__o, __ao, __a | (0 - ((__a) >> 4))));	\
+#define AddAlpha(a) {							\
+	uint8_t   __o = READ(image, __ap);				\
+	uint8_t   __a = (a) + Get4(__o, __ao);				\
+	WRITE(image, __ap, Put4 (__o, __ao, __a | (0 - ((__a) >> 4))));	\
     }
 
 #include "pixman-edge-imp.h"
@@ -100,7 +129,7 @@ clip255 (int x)
 								\
 	while (i__--)						\
 	{							\
-	    WRITE((buf__), clip255 (READ((buf__)) + (val__)));	\
+	    WRITE(image, (buf__), clip255 (READ(image, (buf__)) + (val__)));	\
 	    (buf__)++;						\
 	}							\
     } while (0)
@@ -127,51 +156,55 @@ fbRasterizeEdges8 (pixman_image_t       *image,
     uint32_t  *line;
     int fill_start = -1, fill_end = -1;
     int fill_size = 0;
-    uint32_t *buf = (image)->bits.bits;		
-    int32_t stride = (image)->bits.rowstride;	
-    int32_t width = (image)->bits.width;
-    
+    uint32_t *buf = (image)->bits.bits;
+    int stride = (image)->bits.rowstride;
+    int width = (image)->bits.width;
+
     line = buf + pixman_fixed_to_int (y) * stride;
-    
+
     for (;;)
     {
         uint8_t *ap = (uint8_t *) line;
 	pixman_fixed_t	lx, rx;
 	int	lxi, rxi;
-	
+
 	/* clip X */
 	lx = l->x;
 	if (lx < 0)
 	    lx = 0;
 	rx = r->x;
 	if (pixman_fixed_to_int (rx) >= width)
-	    rx = pixman_int_to_fixed (width);
-	
+	    /* Use the last pixel of the scanline, covered 100%.
+	     * We can't use the first pixel following the scanline,
+	     * because accessing it could result in a buffer overrun.
+	     */
+	    rx = pixman_int_to_fixed (width) - 1;
+
 	/* Skip empty (or backwards) sections */
 	if (rx > lx)
 	{
             int lxs, rxs;
-	    
+
 	    /* Find pixel bounds for span. */
 	    lxi = pixman_fixed_to_int (lx);
 	    rxi = pixman_fixed_to_int (rx);
-	    
+
             /* Sample coverage for edge pixels */
             lxs = RenderSamplesX (lx, 8);
             rxs = RenderSamplesX (rx, 8);
-	    
+
             /* Add coverage across row */
 	    if (lxi == rxi)
 	    {
-		WRITE(ap +lxi, clip255 (READ(ap + lxi) + rxs - lxs));
+		WRITE(image, ap +lxi, clip255 (READ(image, ap + lxi) + rxs - lxs));
 	    }
 	    else
 	    {
-		WRITE(ap + lxi, clip255 (READ(ap + lxi) + N_X_FRAC(8) - lxs));
-		
+		WRITE(image, ap + lxi, clip255 (READ(image, ap + lxi) + N_X_FRAC(8) - lxs));
+
 		/* Move forward so that lxi/rxi is the pixel span */
 		lxi++;
-		
+
 		/* Don't bother trying to optimize the fill unless
 		 * the span is longer than 4 pixels. */
 		if (rxi - lxi > 4)
@@ -209,7 +242,7 @@ fbRasterizeEdges8 (pixman_image_t       *image,
 				add_saturate_8 (ap + lxi, N_X_FRAC(8),
 						fill_start - lxi);
 			    }
-			    
+
 			    /* Update fill_end */
 			    if (rxi < fill_end)
 			    {
@@ -232,21 +265,17 @@ fbRasterizeEdges8 (pixman_image_t       *image,
 		{
 		    add_saturate_8 (ap + lxi, N_X_FRAC(8), rxi - lxi);
 		}
-		
-		/* Do not add in a 0 alpha here. This check is
-		 * necessary to avoid a buffer overrun, (when rx
-		 * is exactly on a pixel boundary). */
-		if (rxs)
-		    WRITE(ap + rxi, clip255 (READ(ap + rxi) + rxs));
+
+		WRITE(image, ap + rxi, clip255 (READ(image, ap + rxi) + rxs));
 	    }
 	}
-	
+
 	if (y == b) {
             /* We're done, make sure we clean up any remaining fill. */
             if (fill_start != fill_end) {
 		if (fill_size == N_Y_FRAC(8))
 		{
-		    MEMSET_WRAPPED (ap + fill_start, 0xff, fill_end - fill_start);
+		    MEMSET_WRAPPED (image, ap + fill_start, 0xff, fill_end - fill_start);
 		}
 		else
 		{
@@ -256,7 +285,7 @@ fbRasterizeEdges8 (pixman_image_t       *image,
             }
 	    break;
         }
-	
+
 	if (pixman_fixed_frac (y) != Y_FRAC_LAST(8))
 	{
 	    RenderEdgeStepSmall (l);
@@ -272,7 +301,7 @@ fbRasterizeEdges8 (pixman_image_t       *image,
             {
 		if (fill_size == N_Y_FRAC(8))
 		{
-		    MEMSET_WRAPPED (ap + fill_start, 0xff, fill_end - fill_start);
+		    MEMSET_WRAPPED (image, ap + fill_start, 0xff, fill_end - fill_start);
 		}
 		else
 		{
@@ -313,7 +342,7 @@ PIXMAN_RASTERIZE_EDGES (pixman_image_t *image,
 
 #ifndef PIXMAN_FB_ACCESSORS
 
-void
+PIXMAN_EXPORT void
 pixman_rasterize_edges (pixman_image_t *image,
 			pixman_edge_t	*l,
 			pixman_edge_t	*r,
